==> ozone-1.1.0-src.tar.gz: ozone-1.1.0-src/hadoop-hdds/server-scm/src/main/java/org/apache/hadoop/hdds/scm/pipeline/choose/algorithms/HealthyPipelineChoosePolicy.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership.  The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.apache.hadoop.hdds.scm.pipeline.choose.algorithms;

import org.apache.hadoop.hdds.scm.PipelineRequestInformation;
import org.apache.hadoop.hdds.scm.pipeline.Pipeline;

import java.util.List;

/**
 * The healthy pipeline choose policy that chooses pipeline
 * until return healthy pipeline.
 */
public class HealthyPipelineChoosePolicy extends RandomPipelineChoosePolicy {

  @Override
  public Pipeline choosePipeline(List<Pipeline> pipelineList,
      PipelineRequestInformation pri) {
    Pipeline fallback = null;
    while (pipelineList.size() > 0) {
      Pipeline pipeline = super.choosePipeline(pipelineList, pri);
      if (pipeline.isHealthy()) {
        return pipeline;
      } else {
        fallback = pipeline;
        pipelineList.remove(pipeline);
      }
    }
    return fallback;
  }
}

==> ozone-1.4.1-src.tar.gz: ozone-1.4.1-src/hadoop-hdds/server-scm/src/main/java/org/apache/hadoop/hdds/scm/pipeline/choose/algorithms/HealthyPipelineChoosePolicy.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership.  The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.apache.hadoop.hdds.scm.pipeline.choose.algorithms;

import org.apache.hadoop.hdds.scm.PipelineChoosePolicy;
import org.apache.hadoop.hdds.scm.PipelineRequestInformation;
import org.apache.hadoop.hdds.scm.pipeline.Pipeline;

import java.util.ArrayList;
import java.util.List;

/**
 * The healthy pipeline choose policy that chooses pipeline
 * until return healthy pipeline.
 */
public class HealthyPipelineChoosePolicy implements PipelineChoosePolicy {

  private PipelineChoosePolicy randomPolicy = new RandomPipelineChoosePolicy();

  @Override
  public Pipeline choosePipeline(List<Pipeline> pipelineList,
      PipelineRequestInformation pri) {
    Pipeline fallback = null;
    while (pipelineList.size() > 0) {
      Pipeline pipeline = randomPolicy.choosePipeline(pipelineList, pri);
      if (pipeline.isHealthy()) {
        return pipeline;
      } else {
        fallback = pipeline;
        pipelineList.remove(pipeline);
      }
    }
    return fallback;
  }

  @Override
  public int choosePipelineIndex(List<Pipeline> pipelineList,
      PipelineRequestInformation pri) {
    List<Pipeline> mutableList = new ArrayList<>(pipelineList);
    Pipeline pipeline = choosePipeline(mutableList, pri);
    return pipelineList.indexOf(pipeline);
  }
}
